import express from 'express';

import SampleValidate from './validate/sample';
import NoteFieldValidate from './validate/note_field';
import SampleModel from '../models/sample'
import MaterialModel from '../models/material';
import NoteModel from '../models/note';
import NoteFieldModel from '../models/note_field';



const router = express.Router();

router.get('/samples', (req, res, next) => {
  if (!req.auth(res, ['read', 'write', 'maintain', 'dev', 'admin'], 'all')) return;

  SampleModel.find({}).lean().exec((err, data) => {
    if (err) return next(err);
    res.json(data.map(e => SampleValidate.output(e)).filter(e => e !== null));  // validate all and filter null values from validation errors
  })
});


router.post('/sample/new', (req, res, next) => {
  if (!req.auth(res, ['write', 'maintain', 'dev', 'admin'], 'basic')) return;

  const {error, value: sample} = SampleValidate.input(req.body, 'new');
  if (error) {
    return res.status(400).json({status: 'Invalid body format'});
  }

  MaterialModel.findById(sample.material_id).lean().exec((err, data: any) => {  // validate material_id
    if (err) return next(err);
    if (!data) {  // could not find material_id
      return res.status(400).json({status: 'Material not available'});
    }
    if (!data.numbers.find(e => e.color === sample.color)) {  // color for material not specified
      return res.status(400).json({status: 'Color not available for material'});
    }
    SampleModel.findOne({number: sample.number}).lean().exec((err, data) => {  // validate sample number
      if (err) return next(err);
      if (data) {  // found entry with sample number
        return res.status(400).json({status: 'Sample number already taken'});
      }

      if (sample.notes.sample_references.length > 0) {  // validate sample_references
        let referencesCount = sample.notes.sample_references.length;
        sample.notes.sample_references.forEach(reference => {
          SampleModel.findById(reference.id).lean().exec((err, data) => {
            if (err) return next(err);
            if (!data) {
              return res.status(400).json({status: 'Sample reference not available'});
            }
            referencesCount --;
            if (referencesCount <= 0) {
              f();
            }
          });
        });
      }
      else {
        f();
      }

      if (sample.notes.hasOwnProperty('custom_fields') && Object.keys(sample.notes.custom_fields).length > 0) {
        customFieldsAdd(Object.keys(sample.notes.custom_fields));
      }

      function f() {  // to resolve async
        new NoteModel(sample.notes).save((err, data) => {
          if (err) return next(err);
          delete sample.notes;
          sample.note_id = data._id;
          sample.user_id = req.authDetails.id;
          new SampleModel(sample).save((err, data) => {
            if (err) return next(err);
            res.json(SampleValidate.output(data.toObject()));
          });
        });
      }
    });
  })
});

router.get('/sample/notes/fields', (req, res, next) => {
  if (!req.auth(res, ['read', 'write', 'maintain', 'dev', 'admin'], 'all')) return;

  NoteFieldModel.find({}).lean().exec((err, data) => {
    if (err) return next(err);
    res.json(data.map(e => NoteFieldValidate.output(e)).filter(e => e !== null));  // validate all and filter null values from validation errors
  })
});


module.exports = router;


function customFieldsAdd (fields) {
  fields.forEach(field => {
    NoteFieldModel.findOneAndUpdate({name: field}, {$inc: {qty: 1}}).lean().exec((err, data) => {  // check if field exists
      if (err) return console.error(err);
      if (!data) {  // new field
        new NoteFieldModel({name: field, qty: 1}).save(err => {
          if (err) return console.error(err);
        })
      }
    });
  });
}